package com.ibm.sysmgt.raidmgr.cim.provider.association;

import com.ibm.sysmgt.raidmgr.cim.provider.instance.*;

import java.io.*;
import java.util.*;
import java.rmi.RemoteException;

import com.sun.wbem.cim.*;
import com.sun.wbem.provider20.AssociatorProvider;

import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.PhysicalDeviceTypeFilter;

public class RAIDRealizesSAFTEChipAssociator extends RAIDAssociatorProviderBase
{
   public RAIDRealizesSAFTEChipAssociator()
   {
      super("RAIDRealizesSAFTEChipAssociator:", "IBMPSG_RAIDRealizesSAFTEChip");
      associationLog.println(logHeader + "RAIDRealizesSAFTEChipAssociator()");
   }

   public Vector associatorHelper(CIMObjectPath assocName, CIMObjectPath objectName, String resultClass, 
                                 String role, String resultRole) throws CIMException
   {
      if (objectName.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_CREATION_CLASS_NAME /* IBMPSG_EnclosureSAFTEDevice */))
      {
         associationLog.println(logHeader + "(associatorHelper) Getting associatorNames for " + EnclosureDevice.CIM_CREATION_CLASS_NAME);

         Vector instances = new Vector();

         // IBMPSG_EnclosureSAFTEDevice plays the ANTECEDENT role only
         if ((role != null) && (!role.equalsIgnoreCase("")) && (!role.equalsIgnoreCase(ANTECEDENT)))
         {
            associationLog.println("!!!" + logHeader + EnclosureDevice.CIM_CREATION_CLASS_NAME + " does not play the " + role + " role");
            return instances;
         }

         CIMObjectPath tempOP = new CIMObjectPath(EnclosureDevice.CIM_SAFTE_CHIP_CLASS_NAME);
         tempOP.setNameSpace(assocName.getNameSpace());
         CIMClass cc = cimom.getClass(tempOP, false);

         String deviceID = (String)getKeyValue(objectName, "DeviceID");
         int enclosureID = Integer.parseInt(getToken(deviceID, ":", "EnclosureID"));
         int controllerID = Adapter.displayIDToAdapterID(getToken(deviceID, ":", "ControllerID"));
         int channelID = Channel.displayIDToChannelID(getToken(deviceID, ":", "ChannelID"));

         RaidSystem system = null;
         try
         {
            system = (RaidSystem)getRaidSystem();
         }
         catch (RemoteException e)
         {
            associationLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
            throw new CIMException(CIMException.CIM_ERR_FAILED);
         }

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         for (Enumeration e = system.getPhysicalDeviceCollection(new PhysicalDeviceTypeFilter(Constants.ENCLOSURE)).elements(); e.hasMoreElements();)
         {
            EnclosureDevice enclosure = (EnclosureDevice)e.nextElement();
            if (enclosure.getChannelID() == channelID && enclosure.getDeviceID() == enclosureID)
            {
               CIMInstance enclosureInstance = enclosure.createCIMSAFTEChipInstance(cc.newInstance());
               instances.addElement(enclosureInstance);
            }
         }

         associationLog.println(logHeader + "associatorHelper returning " + instances.size() + " instances.");
         return instances;
      }
      else if (objectName.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_SAFTE_CHIP_CLASS_NAME /* IBMPSG_SAFTEChip */))
      {
         Vector instances = new Vector();

         associationLog.println(logHeader + "(associatorHelper) Getting associatorNames for " + EnclosureDevice.CIM_SAFTE_CHIP_CLASS_NAME);

         // IBMPSG_SAFTEChip plays the DEPENDENT role only
         if ((role != null) && (!role.equalsIgnoreCase("")) && (!role.equalsIgnoreCase(DEPENDENT)))
         {
            associationLog.println("!!!" + logHeader + EnclosureDevice.CIM_SAFTE_CHIP_CLASS_NAME + " does not play the " + role + " role");
            return instances;
         }

         CIMObjectPath tempOP = new CIMObjectPath(EnclosureDevice.CIM_CREATION_CLASS_NAME);
         tempOP.setNameSpace(assocName.getNameSpace());
         CIMClass cc = cimom.getClass(tempOP, false);

         String tag = (String)getKeyValue(objectName, "Tag");
         int enclosureID = Integer.parseInt(getToken(tag, ":", "EnclosureID"));
         int controllerID = Adapter.displayIDToAdapterID(getToken(tag, ":", "ControllerID"));
         int channelID = Channel.displayIDToChannelID(getToken(tag, ":", "ChannelID"));

         RaidSystem system = null;
         try
         {
            system = (RaidSystem)getRaidSystem();
         }
         catch (RemoteException e)
         {
            associationLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
            throw new CIMException(CIMException.CIM_ERR_FAILED);
         }

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         for (Enumeration e = system.getPhysicalDeviceCollection(new PhysicalDeviceTypeFilter(Constants.ENCLOSURE)).elements(); e.hasMoreElements();)
         {
            EnclosureDevice enclosure = (EnclosureDevice)e.nextElement();
            if (enclosure.getChannelID() == channelID && enclosure.getDeviceID() == enclosureID)
            {
               CIMInstance enclosureInstance = enclosure.createCIMInstance(cc.newInstance());
               instances.addElement(enclosureInstance);
            }
         }

         associationLog.println(logHeader + "associatorHelper returning " + instances.size() + " instances.");
         return instances;
      }

      associationLog.println("!!!" + logHeader + "associatorHelper found no matches, throwing exception.");

      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }

   public Vector referenceHelper(CIMObjectPath assocName, CIMObjectPath objectName, String role) throws CIMException
   {
      if (objectName.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_CREATION_CLASS_NAME /* IBMPSG_EnclosureSAFTEDevice */))
      {
         Vector instances = new Vector();

         associationLog.println(logHeader + "(referenceHelper) Getting referenceNames for " + EnclosureDevice.CIM_CREATION_CLASS_NAME);         

         // IBMPSG_EnclosureSAFTEDevice plays the ANTECEDENT role only
         if ((role != null) && (!role.equalsIgnoreCase("")) && (!role.equalsIgnoreCase(ANTECEDENT)))
         {
            associationLog.println("!!!" + logHeader + EnclosureDevice.CIM_CREATION_CLASS_NAME + " does not play the " + role + " role");
            return instances;
         }

         CIMClass cc = cimom.getClass(assocName, false);

         String deviceID = (String)getKeyValue(objectName, "DeviceID");
         int enclosureID = Integer.parseInt(getToken(deviceID, ":", "EnclosureID"));
         int controllerID = Adapter.displayIDToAdapterID(getToken(deviceID, ":", "ControllerID"));
         int channelID = Channel.displayIDToChannelID(getToken(deviceID, ":", "ChannelID"));

         RaidSystem system = null;
         try
         {
            system = (RaidSystem)getRaidSystem();
         }
         catch (RemoteException e)
         {
            associationLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
            throw new CIMException(CIMException.CIM_ERR_FAILED);
         }

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         for (Enumeration e = system.getPhysicalDeviceCollection(new PhysicalDeviceTypeFilter(Constants.ENCLOSURE)).elements(); e.hasMoreElements();)
         {
            EnclosureDevice enclosure = (EnclosureDevice)e.nextElement();
            if (enclosure.getChannelID() == channelID && enclosure.getDeviceID() == enclosureID)
            {
               CIMInstance enclosureInstance = enclosure.createRealizesSAFTEChipAssociation(cc.newInstance());
               instances.addElement(enclosureInstance);
            }
         }

         associationLog.println(logHeader + "referenceHelper returning " + instances.size() + " instances.");

         return instances;
      }
      else if (objectName.getObjectName().equalsIgnoreCase(EnclosureDevice.CIM_SAFTE_CHIP_CLASS_NAME /* IBMPSG_SAFTEChip */))
      {
         Vector instances = new Vector();

         associationLog.println(logHeader + "(referenceHelper) Getting referenceNames for " + EnclosureDevice.CIM_SAFTE_CHIP_CLASS_NAME);

         // IBMPSG_SAFTEChip plays the DEPENDENT role only
         if ((role != null) && (!role.equalsIgnoreCase("")) && (!role.equalsIgnoreCase(DEPENDENT)))
         {
            associationLog.println("!!!" + logHeader + EnclosureDevice.CIM_SAFTE_CHIP_CLASS_NAME + " does not play the " + role + " role");
            return instances;
         }

         CIMClass cc = cimom.getClass(assocName, false);

         String tag = (String)getKeyValue(objectName, "Tag");
         int enclosureID = Integer.parseInt(getToken(tag, ":", "DeviceID"));
         int controllerID = Adapter.displayIDToAdapterID(getToken(tag, ":", "ControllerID"));
         int channelID = Channel.displayIDToChannelID(getToken(tag, ":", "ChannelID"));

         RaidSystem system = null;
         try
         {
            system = (RaidSystem)getRaidSystem();
         }
         catch (RemoteException e)
         {
            associationLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
            throw new CIMException(CIMException.CIM_ERR_FAILED);
         }

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         for (Enumeration e = system.getPhysicalDeviceCollection(new PhysicalDeviceTypeFilter(Constants.ENCLOSURE)).elements(); e.hasMoreElements();)
         {
            EnclosureDevice enclosure = (EnclosureDevice)e.nextElement();
            if (enclosure.getChannelID() == channelID && enclosure.getDeviceID() == enclosureID)
            {
               CIMInstance enclosureInstance = enclosure.createRealizesSAFTEChipAssociation(cc.newInstance());
               instances.addElement(enclosureInstance);
            }
         }

         associationLog.println(logHeader + "referenceHelper returning " + instances.size() + " instances.");

         return instances;
      }

      associationLog.println("!!!" + logHeader + "referenceHelper found no matches, throwing exception.");

      throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
   }

   //**************************************************************************************************************
   // Now for the InstanceProvider implementation (for Windows)
   //**************************************************************************************************************

   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc, boolean localOnly) throws CIMException
   {
      associationLog.println(logHeader + "enumInstances(" + op.getObjectName() + ")");

      if (!op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
      {
         associationLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }

      SetInteractiveWSAD();

      Vector instances = new Vector();

      RaidSystem system = null;
      try
      {
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         associationLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      for (Enumeration e = system.getPhysicalDeviceCollection(new PhysicalDeviceTypeFilter(Constants.ENCLOSURE)).elements(); e.hasMoreElements();)
      {
         EnclosureDevice enclosure = (EnclosureDevice)e.nextElement();
         instances.addElement(enclosure.createRealizesSAFTEChipAssociation(cc.newInstance()));
      }

      RestoreInteractiveWSAD();

      return instances;
   }

   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc) throws CIMException
   {
      associationLog.println(logHeader + "enumInstances(" + op.getObjectName() + ") Names Only");

      if (!op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
      {
         associationLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());         
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }

      SetInteractiveWSAD();

      Vector cops = new Vector();
      Vector instances = null;

      RaidSystem system = null;
      try
      {
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         associationLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      for (Enumeration e = system.getPhysicalDeviceCollection(new PhysicalDeviceTypeFilter(Constants.ENCLOSURE)).elements(); e.hasMoreElements();)
      {
         EnclosureDevice enclosure = (EnclosureDevice)e.nextElement();
         instances.addElement(enclosure.createRealizesSAFTEChipAssociation(cc.newInstance()));
      }

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         CIMInstance ci = (CIMInstance)e.nextElement();
         CIMObjectPath cop = new CIMObjectPath(cc.getName(), op.getNameSpace());
         cop.setKeys(ci.getKeyValuePairs());
         cops.addElement(cop);
      }

      RestoreInteractiveWSAD();
      return cops;
   }
}
