package com.ibm.sysmgt.raidmgr.cim.provider.instance;

import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.cim.provider.*;

import com.ibm.sysmgt.cim.cimfac.CIMOMHandle;

import com.sun.wbem.cim.*;
import com.sun.wbem.provider.CIMProvider;
import com.sun.wbem.provider20.Authorizable;
import com.sun.wbem.provider20.InstanceProvider;
import com.sun.wbem.client.ProviderCIMOMHandle;
import com.sun.wbem.query.SelectExp;

import java.util.*;
import java.io.*;
import java.rmi.RemoteException;

public class RAIDDiskDriveProvider extends RAIDInstanceProviderBase
{
   public RAIDDiskDriveProvider()
   {
      super("RAIDDiskDriveProvider:");
      instanceLog.println(logHeader + "RAIDDiskDriveProvider()");
   }

   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ") Names Only");

      SetInteractiveWSAD();

      Vector cops = new Vector();
      Vector instances = null;

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(HardDrive.CIM_CREATION_CLASS_NAME))
         instances = enumDiskDrives(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(HardDrive.CIM_PHYSICAL_DISK_DRIVECLASS_NAME))
         instances = enumPhysicalDiskDrives(system, cc);
      else
      {
         instanceLog.println(logHeader + "Does not provide names of class " + op.getObjectName());
         RestoreInteractiveWSAD();
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         CIMInstance ci = (CIMInstance)e.nextElement();
         CIMObjectPath cop = new CIMObjectPath(cc.getName(), op.getNameSpace());
         cop.setKeys(ci.getKeyValuePairs());
         cops.addElement(cop);
      }

      RestoreInteractiveWSAD();
      return cops;
   }

   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc, boolean localOnly) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ")");

      SetInteractiveWSAD();

      if (isConnectedToAgent() == false)
      {
         instanceLog.println(logHeader + "Cannot enumerate classes, not connected to agent.");
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(HardDrive.CIM_CREATION_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return enumDiskDrives(system, cc);
      }         
      if (op.getObjectName().equalsIgnoreCase(HardDrive.CIM_PHYSICAL_DISK_DRIVECLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return enumPhysicalDiskDrives(system, cc);
      }         
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());
         RestoreInteractiveWSAD();
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }
   }

   public CIMInstance getInstance(CIMObjectPath op, CIMClass cc, boolean localOnly) throws CIMException
   {
      instanceLog.println(logHeader + "getInstance(" + op.getObjectName() + ")");

      SetInteractiveWSAD();

      if (isConnectedToAgent() == false)
      {
         instanceLog.println(logHeader + "Can't get class of type "  + op.getObjectName() + ", not connected to agent.");
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      RaidSystem system = null;
      try
      {
         Object[] args = null;
         system = getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(HardDrive.CIM_CREATION_CLASS_NAME))
      {      
         RestoreInteractiveWSAD();
         return getDiskDrive(system, cc, op);
      }
      if (op.getObjectName().equalsIgnoreCase(HardDrive.CIM_PHYSICAL_DISK_DRIVECLASS_NAME))
      {      
         RestoreInteractiveWSAD();
         return getPhysicalDiskDrive(system, cc, op);
      }
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());
         RestoreInteractiveWSAD();
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }      
   }

   // **********************
   // Disk Drives (logical)
   // **********************
   public Vector enumDiskDrives(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {
         Adapter adapter = (Adapter)e.nextElement();
         for (Enumeration e2 = adapter.enumerateHardDrives(); e2.hasMoreElements();)
         {
            HardDrive drive = (HardDrive)e2.nextElement();
            if (drive.getState() != Constants.EMP)
               instances.addElement(drive.createCIMInstance(cc.newInstance()));
         }         
      }

      return instances;
   }   

   public CIMInstance getDiskDrive(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      try
      {
         String deviceID = (String)getKeyValue(op, "DeviceID");
         int diskDriveID = Integer.parseInt(getToken(deviceID, ":", "DiskDriveID"));
         int controllerID = Adapter.displayIDToAdapterID(getToken(deviceID, ":", "ControllerID"));
         int channelID = Channel.displayIDToChannelID(getToken(deviceID, ":", "ChannelID"));

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         for (Enumeration e = adapter.enumerateHardDrives(); e.hasMoreElements();)
         {
            HardDrive drive = (HardDrive)e.nextElement();
            if (drive.getChannelID() == channelID && drive.getDeviceID() == diskDriveID)
               return drive.createCIMInstance(cc.newInstance());
         }
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }

      throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
   }

   // **********************
   // Disk Drives (physical)
   // **********************
   public Vector enumPhysicalDiskDrives(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {
         Adapter adapter = (Adapter)e.nextElement();
         for (Enumeration e2 = adapter.enumerateHardDrives(); e2.hasMoreElements();)
         {
            HardDrive drive = (HardDrive)e2.nextElement();
            if (drive.getState() != Constants.EMP)
               instances.addElement(drive.createCIMPhysicalDiskDriveInstance(cc.newInstance()));
         }         
      }

      return instances;
   }   

   public CIMInstance getPhysicalDiskDrive(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      try
      {
         String tag = (String)getKeyValue(op, "Tag");         

         int controller = Adapter.displayIDToAdapterID(getToken(tag, ":", "ControllerID"));
         int channelID = Channel.displayIDToChannelID(getToken(tag, ":", "ChannelID"));
         int deviceID = Integer.parseInt(getToken(tag, ":", "DeviceID"));

         Adapter adapter = (Adapter)system.getAdapter(controller);
         for (Enumeration e = adapter.enumerateHardDrives(); e.hasMoreElements();)
         {
            HardDrive drive = (HardDrive)e.nextElement();
            if (drive.getChannelID() == channelID && drive.getDeviceID() == deviceID)
               return drive.createCIMPhysicalDiskDriveInstance(cc.newInstance());
         }
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(log);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }

      throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
   }

    public static void main(String [] args)
    {
        RAIDDiskDriveProvider provider = new RAIDDiskDriveProvider();
        Object [] args1 = null;
        try
        {
            RaidSystem system = provider.getRaidSystem();
            System.out.println(system);
            System.out.println("Controller count = " + system.getAdapterCount());
            System.exit(0);
        }
        catch (RemoteException ex)
        {
            System.out.println("Error accessing the ServeRAID Manager agent. Exception: " + ex);
        }
    }
}
